@@ -33,7 +33,6 @@ from .gen_rst import (
     SPHX_GLR_SIG,
     _get_call_memory_and_base,
     _get_callables,
-    _get_class,
     _get_gallery_header,
     generate_dir_rst,
 )
@@ -50,11 +49,14 @@ from .scrapers import _import_matplotlib
 from .sorting import ExplicitOrder
 from .utils import (
     _collect_gallery_files,
+    _combine_backreferences,
     _format_toctree,
     _has_graphviz,
     _has_optipng,
     _has_pypandoc,
+    _read_json,
     _replace_md5,
+    _write_json,
 )
 
 _KNOWN_CSS = (
@@ -94,11 +96,8 @@ DEFAULT_GALLERY_CONF = {
     "reference_url": {},
     "capture_repr": ("_repr_html_", "__repr__"),
     "ignore_repr_types": r"",
-    # Build options
-    # -------------
-    # 'plot_gallery' also accepts strings that evaluate to a bool, e.g. "True",
-    # "False", "1", "0" so that they can be easily set via command line
-    # switches of sphinx-build
+    # 'plot_gallery' should accept strings that evaluate to a bool, to allow
+    # setting via `sphinx-build -D` on command line
     "plot_gallery": "True",
     "download_all_examples": True,
     "abort_on_example_error": False,
@@ -111,6 +110,7 @@ DEFAULT_GALLERY_CONF = {
     "expected_failing_examples": set(),
     "thumbnail_size": (400, 280),  # Default CSS does 0.4 scaling (160, 112)
     "min_reported_time": 0,
+    "write_computation_times": os.getenv("SOURCE_DATE_EPOCH") is None,
     "binder": {},
     "jupyterlite": {},
     "promote_jupyter_magic": False,
@@ -145,7 +145,11 @@ logger = sphinx.util.logging.getLogger("sphinx-gallery")
 
 
 def _bool_eval(x):
-    """Evaluate bool only configs, to allow setting via -D on the command line."""
+    """Evaluate bool only configs, allows setting via `sphinx-build -D` on command line.
+
+    Ensures that strings that evaluate to a bool, e.g. "True", "False", "1", "0",
+    work as expected.
+    """
     if isinstance(x, str):
         try:
             x = eval(x)
@@ -159,9 +163,11 @@ def _update_gallery_conf_exclude_implicit_doc(gallery_conf):
 
     This is separate function for better testability.
     """
-    # prepare regex for exclusions from implicit documentation
+    # prepare regex for exclusions from implicit documentation, ensuring that what
+    # gets complied has a stable __repr__ (i.e., by sorting the exclude_implicit_doc
+    # set before joining)
     exclude_regex = (
-        re.compile("|".join(gallery_conf["exclude_implicit_doc"]))
+        re.compile("|".join(sorted(gallery_conf["exclude_implicit_doc"])))
         if gallery_conf["exclude_implicit_doc"]
         else False
     )
@@ -357,11 +363,18 @@ def _fill_gallery_conf_defaults(sphinx_gallery_conf, app=None, check_keys=True):
     _check_extra_config_keys(gallery_conf, sphinx_gallery_conf, check_keys)
 
     gallery_conf.update(sphinx_gallery_conf)
-    # XXX anything that can only be a bool (rather than str) should probably be
-    # evaluated this way as it allows setting via -D on the command line
+    # Bool only (i.e. not string) configs are evaluated with
+    # `_bool_eval` to allow setting via `sphinx-build -D`
     for key in (
+        "download_all_examples",
+        "inspect_global_variables",
+        "line_numbers",
+        "nested_sections",
+        "only_warn_on_example_error",
         "promote_jupyter_magic",
+        "remove_config_comments",
         "run_stale_examples",
+        "show_signature",
     ):
         gallery_conf[key] = _bool_eval(gallery_conf[key])
 
@@ -491,18 +504,20 @@ def _fill_gallery_conf_defaults(sphinx_gallery_conf, app=None, check_keys=True):
     ):
         raise ConfigError(
             'gallery_conf["show_api_usage"] must be True, False or "unused", '
-            f'got {gallery_conf["show_api_usage"]}'
+            f"got {gallery_conf['show_api_usage']}"
         )
 
-    # classes (not pickleable so need to resolve using fully qualified name)
-    _get_class(gallery_conf, "within_subsection_order")  # make sure it works
+    # check `within_subsection_order`
+    _get_callables(
+        gallery_conf, "within_subsection_order", src_dir=""
+    )  # make sure it works
 
     _update_gallery_conf_exclude_implicit_doc(gallery_conf)
 
     return gallery_conf
 
 
-def get_subsections(srcdir, examples_dir, gallery_conf, check_for_index=True):
+def get_subsections(srcdir, examples_dir, gallery_conf, check_for_header=True):
     """Return the list of subsections of a gallery.
 
     Parameters
@@ -513,8 +528,8 @@ def get_subsections(srcdir, examples_dir, gallery_conf, check_for_index=True):
         path to the examples directory relative to conf.py
     gallery_conf : Dict[str, Any]
         Sphinx-Gallery configuration dictionary.
-    check_for_index : bool
-        only return subfolders contain a GALLERY_HEADER file, default True
+    check_for_header : bool
+        only return subfolders that contain a GALLERY_HEADER file, default True
 
     Returns
     -------
@@ -524,26 +539,32 @@ def get_subsections(srcdir, examples_dir, gallery_conf, check_for_index=True):
     if gallery_conf["subsection_order"] is None:
         sortkey = None
     else:
+        subsec_order = gallery_conf.get("subsection_order")
+        if isinstance(subsec_order, list):
+            gallery_conf["subsection_order"] = ExplicitOrder(subsec_order)
         (sortkey,) = _get_callables(gallery_conf, "subsection_order")
-        if isinstance(sortkey, list):
-            sortkey = ExplicitOrder(sortkey)
     subfolders = [subfolder for subfolder in os.listdir(examples_dir)]
-    if check_for_index:
+    if check_for_header:
         subfolders = [
             subfolder
             for subfolder in subfolders
+            # Return is not `None` only when a gallery head file is found
             if _get_gallery_header(
                 os.path.join(examples_dir, subfolder), gallery_conf, raise_error=False
             )
             is not None
         ]
     else:
-        # just make sure its a directory
+        # just make sure its a directory, that is not `__pycache__`
         subfolders = [
             subfolder
             for subfolder in subfolders
-            if os.path.isdir(os.path.join(examples_dir, subfolder))
+            if (
+                subfolder != "__pycache__"
+                and os.path.isdir(os.path.join(examples_dir, subfolder))
+            )
         ]
+
     base_examples_dir_path = os.path.relpath(examples_dir, srcdir)
     subfolders_with_path = [
         os.path.join(base_examples_dir_path, item) for item in subfolders
@@ -567,6 +588,12 @@ def _prepare_sphx_glr_dirs(gallery_conf, srcdir):
     if not isinstance(gallery_dirs, list):
         gallery_dirs = [gallery_dirs]
 
+    if len(examples_dirs) != len(gallery_dirs):
+        logger.warning(
+            "'examples_dirs' and 'gallery_dirs' are of different lengths. "
+            "Surplus entries will be ignored."
+        )
+
     if bool(gallery_conf["backreferences_dir"]):
         backreferences_dir = os.path.join(srcdir, gallery_conf["backreferences_dir"])
         os.makedirs(backreferences_dir, exist_ok=True)
@@ -608,13 +635,15 @@ def _finish_index_rst(
         )
         indexst += subsections_toctree
 
-    if sg_root_index:
-        # Download examples
-        if gallery_conf["download_all_examples"]:
-            download_fhindex = generate_zipfiles(
-                gallery_dir_abs_path, app.builder.srcdir, gallery_conf
-            )
+    # Always generate download zipfiles, only add to index.rst if required
+    if gallery_conf["download_all_examples"]:
+        download_fhindex = generate_zipfiles(
+            gallery_dir_abs_path, app.builder.srcdir, gallery_conf
+        )
+        if sg_root_index:
             indexst += download_fhindex
+
+    if sg_root_index:
         # Signature
         if app.config.sphinx_gallery_conf["show_signature"]:
             indexst += SPHX_GLR_SIG
@@ -648,7 +677,7 @@ def _build_recommender(gallery_conf, gallery_dir_abs_path, subsecs):
                 # NOTE we don't take account of `ignore_pattern` and ignore
                 # ext in `example_extensions`
                 [fname for fname in Path(src_dir).iterdir() if fname.suffix == ".py"],
-                key=_get_class(gallery_conf, "within_subsection_order")(src_dir),
+                key=_get_callables(gallery_conf, "within_subsection_order", src_dir)[0],
             )
             gallery_py_files.append(
                 [os.path.join(src_dir, fname) for fname in py_files]
@@ -719,6 +748,8 @@ def generate_gallery_rst(app):
     examples_dirs = [ex_dir for ex_dir, _ in workdirs]
     _collect_gallery_files(examples_dirs, gallery_conf, check_filenames=True)
 
+    backrefs_all = {}
+
     for examples_dir, gallery_dir in workdirs:
         examples_dir_abs_path = os.path.join(app.builder.srcdir, examples_dir)
         gallery_dir_abs_path = os.path.join(app.builder.srcdir, gallery_dir)
@@ -730,6 +761,7 @@ def generate_gallery_rst(app):
             this_content,
             this_costs,
             this_toctree_items,
+            backrefs_root,
         ) = generate_dir_rst(
             examples_dir_abs_path,
             gallery_dir_abs_path,
@@ -738,11 +770,15 @@ def generate_gallery_rst(app):
             is_subsection=False,
         )
 
-        # `this_context` is None when user provides own index.rst
+        _combine_backreferences(backrefs_all, backrefs_root)
+
+        # `this_content` is None when user provides own index.rst
         sg_root_index = this_content is not None
         costs += this_costs
         write_computation_times(gallery_conf, gallery_dir_abs_path, this_costs)
 
+        # `indexst` variable must exist, as passed to `_finish_index_rst`
+        indexst = ""
         # Create root gallery index.rst
         if sg_root_index:
             # :orphan: to suppress "not included in TOCTREE" sphinx warnings
@@ -758,7 +794,7 @@ def generate_gallery_rst(app):
             app.builder.srcdir,
             examples_dir_abs_path,
             gallery_conf,
-            check_for_index=sg_root_index,
+            check_for_header=sg_root_index,
         )
         for subsection in subsecs:
             src_dir = os.path.join(examples_dir_abs_path, subsection)
@@ -774,8 +810,11 @@ def generate_gallery_rst(app):
                 subsection_index_content,
                 subsection_costs,
                 subsection_toctree_filenames,
+                backrefs_subsec,
             ) = generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs)
 
+            _combine_backreferences(backrefs_all, backrefs_subsec)
+
             has_subsection_header = False
             if subsection_index_content:
                 # Filter tags to prevent tag duplication across the documentation
@@ -815,6 +854,17 @@ def generate_gallery_rst(app):
     # Write a single global sg_execution_times
     write_computation_times(gallery_conf, None, costs)
 
+    # Write backreferences_all to file
+    if gallery_conf["backreferences_dir"]:
+        _write_json(
+            Path(
+                gallery_conf["src_dir"],
+                gallery_conf["backreferences_dir"],
+                "backreferences_all",
+            ),
+            backrefs_all,
+        )
+
     if gallery_conf["show_api_usage"] is not False:
         _init_api_usage(app.builder.srcdir)
     _finalize_backreferences(seen_backrefs, gallery_conf)
@@ -898,8 +948,8 @@ def _format_for_writing(costs, *, src_dir, kind="rst"):
         else:  # like in generate_gallery
             assert kind == "console"
             name = rel_path
-            t = f'{cost["t"]:0.2f} sec'
-        m = f'{cost["mem"]:.1f} MB'
+            t = f"{cost['t']:0.2f} sec"
+        m = f"{cost['mem']:.1f} MB"
         lines.append([name, t, m])
     lens = [max(x) for x in zip(*[[len(item) for item in cost] for cost in lines])]
     return lines, lens
@@ -917,6 +967,8 @@ def write_computation_times(gallery_conf, target_dir, costs):
     costs: List[Dict]
         List of dicts of computation costs and paths, see gen_rst.py for details.
     """
+    if not gallery_conf["write_computation_times"]:
+        return
     total_time = sum(cost["t"] for cost in costs)
     if target_dir is None:  # all galleries together
         out_dir = gallery_conf["src_dir"]
@@ -927,7 +979,7 @@ def write_computation_times(gallery_conf, target_dir, costs):
         out_dir = target_dir
         where = os.path.relpath(target_dir, gallery_conf["src_dir"])
         kind = "rst"
-        ref_extra = f'{where.replace(os.sep, "_")}_'
+        ref_extra = f"{where.replace(os.sep, '_')}_"
     new_ref = f"sphx_glr_{ref_extra}sg_execution_times"
     out_file = Path(out_dir) / "sg_execution_times.rst"
     if out_file.is_file() and total_time == 0:  # a re-run
@@ -1118,7 +1170,7 @@ def _make_graph(fname, entries, gallery_conf):
                 dg.node(node_to, **node_kwargs)
                 dg.edge(node_from, node_to, color=API_COLORS["edge"])
         # add modules with all API entries
-        for module in gallery_conf["_sg_api_entries"]["module"]:
+        for module in gallery_conf["_sg_api_entries"].get("module", []):
             struct = module.split(".")
             for i in range(len(struct) - 1):
                 if struct[i + 1] not in lut:
@@ -1204,24 +1256,23 @@ def write_api_entry_usage(app, docname, source):
     # find used and unused API entries
     unused_api_entries = list()
     used_api_entries = dict()
+    backreferences_all = _read_json(Path(backreferences_dir, "backreferences_all.json"))
+    src_dir = gallery_conf["src_dir"]
     for entry in example_files:
         # don't include built-in methods etc.
         if re.match(gallery_conf["api_usage_ignore"], entry) is not None:
             continue
         # check if backreferences empty
-        example_fname = os.path.join(backreferences_dir, f"{entry}.examples.new")
-        if not os.path.isfile(example_fname):  # use without new
-            example_fname = os.path.splitext(example_fname)[0]
-        assert os.path.isfile(example_fname)
-        if os.path.getsize(example_fname) == 0:
+        backref_entry = backreferences_all.get(entry, None)
+        if backref_entry is None:
             unused_api_entries.append(entry)
         else:
             used_api_entries[entry] = list()
-            with open(example_fname, encoding="utf-8") as fid2:
-                for line in fid2:
-                    if line.startswith("  :ref:"):
-                        example_name = line.split("`")[1]
-                        used_api_entries[entry].append(example_name)
+            for br in backref_entry:
+                # br[2] = abs path to target directory
+                example_path = Path(br[2], br[0]).relative_to(src_dir)
+                ref_name = str(example_path).replace(os.sep, "_")
+                used_api_entries[entry].append(f"sphx_glr_{ref_name}")
 
     for entry in sorted(unused_api_entries):
         source[0] += f"- :{get_entry_type(entry)}:`{entry}`\n"
@@ -1339,8 +1390,7 @@ def write_junit_xml(gallery_conf, target_dir, costs):
             continue  # not subselected by our regex
         title = gallery_conf["titles"][fname]
         output += (
-            '<testcase classname={!s} file={!s} line="1" '
-            'name={!s} time="{!r}">'.format(
+            '<testcase classname={!s} file={!s} line="1" name={!s} time="{!r}">'.format(
                 quoteattr(os.path.splitext(os.path.basename(fname))[0]),
                 quoteattr(os.path.relpath(fname, src_dir)),
                 quoteattr(title),
