@@ -32,7 +32,7 @@ import sphinx.util
 from sphinx.errors import ConfigError, ExtensionError
 from sphinx.util.console import blue, bold, red
 
-from . import glr_path_static, py_source_parser
+from . import glr_path_static, py_source_parser, rst_source_parser
 from .backreferences import (
     THUMBNAIL_PARENT_DIV,
     THUMBNAIL_PARENT_DIV_CLOSE,
@@ -42,7 +42,6 @@ from .backreferences import (
     identify_names,
 )
 from .block_parser import BlockParser
-from .docs_resolv import _write_code_obj
 from .interactive_example import (
     _add_jupyterlite_badge_logo,
     gen_binder_rst,
@@ -60,9 +59,12 @@ from .scrapers import (
 from .utils import (
     _W_KW,
     _collect_gallery_files,
+    _combine_backreferences,
     _format_toctree,
     _replace_md5,
+    _write_json,
     get_md5sum,
+    iter_gallery_header_filenames,
     optipng,
     scale_image,
     status_iterator,
@@ -284,8 +286,12 @@ def _sanitize_rst(string):
     # ``whatever thing`` --> whatever thing
     p = r"(\s|^)`"
     string = re.sub(p + r"`([^`]+)`" + e, r"\1\2\3", string)
+    # `~mymodule.MyClass` --> MyClass
+    string = re.sub(p + r"~([^`]+)" + e, _regroup, string)
+
     # `whatever thing` --> whatever thing
-    string = re.sub(p + r"([^`]+)" + e, r"\1\2\3", string)
+    # `.MyClass` --> MyClass
+    string = re.sub(p + r"\.?([^`]+)" + e, r"\1\2\3", string)
 
     # **string** --> string
     string = re.sub(r"\*\*([^\*]*)\*\*", r"\1", string)
@@ -439,13 +445,19 @@ def _get_gallery_header(dir_, gallery_conf, raise_error=True):
         if os.path.isfile(fpth):
             return None
     # Next look for GALLERY_HEADER.[ext] (and for backward-compatibility README.[ext]
-    extensions = [".txt"] + sorted(gallery_conf["source_suffix"])
-    for ext in extensions:
-        for fname in ("GALLERY_HEADER", "README", "readme"):
-            fpth = os.path.join(dir_, fname + ext)
-            if os.path.isfile(fpth):
-                return fpth
+    for fname in iter_gallery_header_filenames(gallery_conf):
+        fpth = os.path.join(dir_, fname)
+        if os.path.isfile(fpth):
+            return fpth
     if raise_error:
+        extensions = list(
+            sorted(
+                set(
+                    os.path.splitext(fname)[1]
+                    for fname in iter_gallery_header_filenames(gallery_conf)
+                )
+            )
+        )
         raise ExtensionError(
             "Example directory {} does not have a GALLERY_HEADER file with "
             "one of the expected file extensions {}. Please write one to "
@@ -505,7 +517,7 @@ def generate_dir_rst(
     seen_backrefs,
     is_subsection=True,
 ):
-    """Generate the gallery reStructuredText for an example directory.
+    """Generate output example reST files for one gallery (sub)directory.
 
     Parameters
     ----------
@@ -535,6 +547,11 @@ def generate_dir_rst(
          with keys "t", "mem", "src_file", and "target_dir".
     toctree_items: list,
         List of example file names we generated ReST for.
+    backrefs_dir : dict[str, tuple]
+        Dictionary where value is the backreference object and value
+        is a tuple containing: example filename, full path to example source directory,
+        full path to example target directory, intro, title.
+
     """
     index_content = ""
     # `_get_gallery_header` returns `None` if user supplied `index.rst`
@@ -562,7 +579,7 @@ def generate_dir_rst(
     listdir = _collect_gallery_files([src_dir], gallery_conf)
     # sort them
     sorted_listdir = sorted(
-        listdir, key=_get_class(gallery_conf, "within_subsection_order")(src_dir)
+        listdir, key=_get_callables(gallery_conf, "within_subsection_order", src_dir)[0]
     )
 
     # Add div containing all thumbnails;
@@ -578,6 +595,8 @@ def generate_dir_rst(
         length=len(sorted_listdir),
     )
 
+    backrefs_dir = {}
+
     parallel = list
     p_fun = generate_file_rst
     if gallery_conf["parallel"]:
@@ -621,15 +640,17 @@ def generate_dir_rst(
 
         # Write backreferences
         if "backrefs" in out_vars:
-            _write_backreferences(
+            backrefs_example = _write_backreferences(
                 out_vars["backrefs"],
                 seen_backrefs,
                 gallery_conf,
+                src_dir,
                 target_dir,
                 fname,
                 intro,
                 title,
             )
+            _combine_backreferences(backrefs_dir, backrefs_example)
 
     # Close thumbnail parent div
     index_content += THUMBNAIL_PARENT_DIV_CLOSE
@@ -656,6 +677,7 @@ def generate_dir_rst(
         index_content,
         costs,
         toctree_filenames,
+        backrefs_dir,
     )
 
 
@@ -795,6 +817,9 @@ def _get_parser(fname, gallery_conf):
     if fname.endswith(".py"):
         parser = py_source_parser
         language = "Python"
+    elif fname.endswith(".rst"):
+        parser = rst_source_parser
+        language = "RST"
     else:
         parser = BlockParser(fname, gallery_conf)
         language = parser.language
@@ -884,7 +909,7 @@ def _get_last_repr(capture_repr, ___):
 
 
 def _get_code_output(
-    is_last_expr, example_globals, gallery_conf, logging_tee, images_rst, file_conf
+    is_last_expr, example_globals, gallery_conf, logging_tee, images_rst, capture_repr
 ):
     """Obtain standard output and html output in reST.
 
@@ -917,7 +942,6 @@ def _get_code_output(
         ignore_repr = False
         if gallery_conf["ignore_repr_types"]:
             ignore_repr = re.search(gallery_conf["ignore_repr_types"], str(type(___)))
-        capture_repr = file_conf.get("capture_repr", gallery_conf["capture_repr"])
         if capture_repr != () and not ignore_repr:
             last_repr, repr_meth = _get_last_repr(capture_repr, ___)
 
@@ -996,12 +1020,30 @@ def execute_code_block(
     sys.path.append(os.getcwd())
 
     # Save figures unless there is a `sphinx_gallery_defer_figures` flag
-    match = re.search(
+    defer_figs_match = re.search(
         r"^[\ \t]*#\s*sphinx_gallery_defer_figures[\ \t]*\n?",
         block.content,
         re.MULTILINE,
     )
-    need_save_figures = match is None
+    need_save_figures = defer_figs_match is None
+
+    block_conf = py_source_parser.extract_file_config(block.content)
+
+    # Add `sphinx_gallery_multi_image_block` setting to block variables
+    # (extract config rather than just regex search since the option's value is needed)
+    script_vars["multi_image"] = block_conf.get("multi_image_block")
+
+    # Determine the block's `capture_repr` value by prioritising the block-level
+    # `sphinx_gallery_capture_repr_block` setting, then the file-level
+    # `sphinx_gallery_capture_repr` setting, and finally the
+    # global `capture_repr` gallery setting.
+    capture_repr = block_conf.get(
+        "capture_repr_block",
+        file_conf.get("capture_repr", gallery_conf["capture_repr"]),
+    )
+
+    # Add file_conf to script_vars to be read by image scrapers
+    script_vars["file_conf"] = file_conf
 
     try:
         # The "compile" step itself can fail on a SyntaxError, so just prepend
@@ -1046,7 +1088,7 @@ def execute_code_block(
             gallery_conf,
             logging_tee,
             images_rst,
-            file_conf,
+            capture_repr,
         )
     finally:
         _reset_cwd_syspath(cwd, sys_path)
@@ -1232,10 +1274,18 @@ def _get_backreferences(gallery_conf, script_vars, script_blocks, node, target_f
     ref_regex = _make_ref_regex(gallery_conf["default_role"])
     example_code_obj = identify_names(script_blocks, ref_regex, global_variables, node)
     if example_code_obj:
-        _write_code_obj(target_file, example_code_obj)
+        _write_json(target_file, example_code_obj, ".codeobj")
     exclude_regex = gallery_conf["exclude_implicit_doc_regex"]
+
+    def _normalize_name(cobj):
+        full_name = "{module}.{name}".format(**cobj)
+        for pattern in gallery_conf["prefer_full_module"]:
+            if re.search(pattern, full_name):
+                return full_name
+        return "{module_short}.{name}".format(**cobj)
+
     backrefs = {
-        "{module_short}.{name}".format(**cobj)
+        _normalize_name(cobj)
         for cobjs in example_code_obj.values()
         for cobj in cobjs
         if cobj["module"].startswith(gallery_conf["doc_module"])
@@ -1335,7 +1385,7 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf):
         script_blocks, script_vars, gallery_conf, file_conf
     )
 
-    logger.debug("%s ran in : %.2g seconds\n", src_file, time_elapsed)
+    logger.debug("%s ran in : %.2g seconds", src_file, time_elapsed)
 
     # Create dummy images
     _make_dummy_images(executable, file_conf, script_vars)
@@ -1412,7 +1462,7 @@ EXAMPLE_HEADER = """
         :class: sphx-glr-download-link-note
 
         :ref:`Go to the end <sphx_glr_download_{1}>`
-        to download the full example code.{2}
+        to download the full example code{2}
 
 .. rst-class:: sphx-glr-example-title
 
@@ -1488,7 +1538,14 @@ def rst_blocks(
             block_separator = (
                 "\n\n" if not script_block.content.endswith("\n") else "\n"
             )
-            example_rst += script_block.content + block_separator
+            # Remove "# noqa: E501" at the end of any lines (issue #1403)
+            text_content = re.sub(
+                r"^(.*)( +# noqa: ?E501 *)$",
+                r"\1",  # replace with first capturing group
+                script_block.content,
+                flags=re.MULTILINE,
+            )
+            example_rst += text_content + block_separator
 
     return example_rst
 
@@ -1529,16 +1586,17 @@ def save_rst_example(
     jupyterlite_conf = gallery_conf["jupyterlite"]
     is_jupyterlite_enabled = jupyterlite_conf is not None
 
-    interactive_example_text = ""
     if is_binder_enabled or is_jupyterlite_enabled:
-        interactive_example_text += " or to run this example in your browser via "
+        interactive_example_text = " or to run this example in your browser via "
+    else:
+        interactive_example_text = "."
 
     if is_binder_enabled and is_jupyterlite_enabled:
-        interactive_example_text += "JupyterLite or Binder"
+        interactive_example_text += "JupyterLite or Binder."
     elif is_binder_enabled:
-        interactive_example_text += "Binder"
+        interactive_example_text += "Binder."
     elif is_jupyterlite_enabled:
-        interactive_example_text += "JupyterLite"
+        interactive_example_text += "JupyterLite."
 
     example_rst = (
         EXAMPLE_HEADER.format(example_fname, ref_fname, interactive_example_text)
@@ -1594,46 +1652,32 @@ def save_rst_example(
     _replace_md5(write_file_new, mode="t")
 
 
-def _get_class(gallery_conf, key):
-    """Get a class for the given conf key."""
-    what = f"sphinx_gallery_conf[{repr(key)}]"
-    val = gallery_conf[key]
-    if key == "within_subsection_order":
-        alias = (  # convenience aliases
-            "ExampleTitleSortKey",
-            "FileNameSortKey",
-            "FileSizeSortKey",
-            "NumberOfCodeLinesSortKey",
-        )
-        if val in alias:
-            val = f"sphinx_gallery.sorting.{val}"
-    if isinstance(val, str):
-        if "." not in val:
-            raise ConfigError(
-                f"{what} must be a fully qualified name string, got {val}"
-            )
-        mod, attr = val.rsplit(".", 1)
-        try:
-            val = getattr(importlib.import_module(mod), attr)
-        except Exception:
-            raise ConfigError(
-                f"{what} must be a fully qualified name string, could not import "
-                f"{attr} from {mod}"
-            )
-    if not inspect.isclass(val):
-        raise ConfigError(
-            f"{what} must be 1) a fully qualified name (string) that resolves "
-            f"to a class or 2) or a class itself, got {val.__class__.__name__} "
-            f"({repr(val)})"
-        )
-    return val
+def _get_callables(gallery_conf, key, src_dir=None):
+    """Get callables for the given conf key, returning tuple of callable(s).
 
+    If value is a string, import, with the following exceptions:
 
-def _get_callables(gallery_conf, key):
-    """Get callables for the given conf key, returning tuple of callable(s)."""
-    singletons = ("reset_argv", "minigallery_sort_order", "subsection_order")
+    * `within_subsection_order` - add full path if value is a built-in aliases,
+      instantiate if value is a class
+    * `image_scrapers` - get value from `_scraper_dict` or import module and
+      get `_get_sg_image_scraper` attribute
+    * `reset_modules` - get value from `_reset_dict`
+    """
+    builtin_aliases = (
+        "ExampleTitleSortKey",
+        "FileNameSortKey",
+        "FileSizeSortKey",
+        "NumberOfCodeLinesSortKey",
+    )
+    singletons = (
+        "reset_argv",
+        "minigallery_sort_order",
+        "subsection_order",
+        "within_subsection_order",
+    )
     # the following should be the case (internal use only):
     assert key in ("image_scrapers", "reset_modules", "jupyterlite") + singletons, key
+    # Get correct config value and ensure it's a list
     which = gallery_conf[key]
     if key == "jupyterlite":
         which = [which["notebook_modification_function"]]
@@ -1642,14 +1686,21 @@ def _get_callables(gallery_conf, key):
     if not isinstance(which, (tuple, list)):
         which = [which]
     which = list(which)
+
     for wi, what in enumerate(which):
+        # Get readable string, for error message
         if key == "jupyterlite":
             readable = f"{key}['notebook_modification_function']"
         elif key in singletons:
             readable = f"{key}={repr(what)}"
         else:
             readable = f"{key}[{wi}]={repr(what)}"
+        # Import string FQNs or get value from dict
         if isinstance(what, str):
+            # use fully qualified name to resolve builtin callable classes
+            # (otherwise not serializable)
+            if what in builtin_aliases:
+                what = f"sphinx_gallery.sorting.{what}"
             if "." in what:
                 mod, attr = what.rsplit(".", 1)
                 try:
@@ -1674,6 +1725,10 @@ def _get_callables(gallery_conf, key):
                     raise ConfigError(f"Unknown string option for {readable}: {what}")
                 what = _reset_dict[what]
             which[wi] = what
+        # make sure classes get instantiated (so they become callable)
+        if key == "within_subsection_order" and inspect.isclass(what):
+            what = what(src_dir)
+            which[wi] = what
         if inspect.isclass(what):
             raise ConfigError(
                 f"Got class rather than callable instance for {readable}: {what}"
