@@ -75,8 +75,8 @@ def test_split_code_and_text_blocks():
     )
 
     assert file_conf == {}
-    assert blocks[0][0] == "text"
-    assert blocks[1][0] == "code"
+    assert blocks[0].type == "text"
+    assert blocks[1].type == "code"
 
 
 def test_bug_cases_of_notebook_syntax():
@@ -183,10 +183,10 @@ def test_rst_block_after_docstring(gallery_conf, tmpdir):
 
     assert file_conf == {}
     assert len(blocks) == 4
-    assert blocks[0][0] == "text"
-    assert blocks[1][0] == "text"
-    assert blocks[2][0] == "text"
-    assert blocks[3][0] == "text"
+    assert blocks[0].type == "text"
+    assert blocks[1].type == "text"
+    assert blocks[2].type == "text"
+    assert blocks[3].type == "text"
 
     script_vars = {"execute_script": ""}
     file_conf = {}
@@ -216,6 +216,53 @@ Paragraph 3
     assert example_rst == want_rst
 
 
+def test_rst_block_noqa_removal(gallery_conf, tmpdir):
+    """Check "# noqa: E501" removed from end of text blocks (issue #1403)."""
+    filename = str(tmpdir.join("temp.py"))
+    with open(filename, "w") as f:
+        f.write(
+            "\n".join(
+                [
+                    '"Docstring"',
+                    "####################",
+                    "# Paragraph 1",
+                    "# has a noqa at the end. # noqa: E501",
+                    "",
+                    "#%%",
+                    "# Paragraph 2 also has a noqa # noqa:E501",
+                    "",
+                    "# %%",
+                    "# Paragraph 3",
+                    "",
+                ]
+            )
+        )
+    file_conf, blocks = split_code_and_text_blocks(filename)
+
+    script_vars = {"execute_script": ""}
+    output_blocks, _ = sg.execute_script(blocks, script_vars, gallery_conf, file_conf)
+
+    example_rst = sg.rst_blocks(blocks, output_blocks, file_conf, gallery_conf)
+    want_rst = """\
+Docstring
+
+.. GENERATED FROM PYTHON SOURCE LINES 3-5
+
+Paragraph 1
+has a noqa at the end.
+
+.. GENERATED FROM PYTHON SOURCE LINES 7-8
+
+Paragraph 2 also has a noqa
+
+.. GENERATED FROM PYTHON SOURCE LINES 10-11
+
+Paragraph 3
+
+"""
+    assert example_rst == want_rst
+
+
 def test_rst_empty_code_block(gallery_conf, tmpdir):
     """Test that we can "execute" a code block containing only comments."""
     gallery_conf.update(image_scrapers=())
@@ -228,7 +275,7 @@ def test_rst_empty_code_block(gallery_conf, tmpdir):
                     "####################",
                     "# Paragraph 1",
                     "",
-                    "# just a comment" "",
+                    "# just a comment",
                 ]
             )
         )
@@ -236,9 +283,9 @@ def test_rst_empty_code_block(gallery_conf, tmpdir):
 
     assert file_conf == {}
     assert len(blocks) == 3
-    assert blocks[0][0] == "text"
-    assert blocks[1][0] == "text"
-    assert blocks[2][0] == "code"
+    assert blocks[0].type == "text"
+    assert blocks[1].type == "text"
+    assert blocks[2].type == "code"
 
     gallery_conf["abort_on_example_error"] = True
     script_vars = dict(
@@ -288,8 +335,8 @@ b = 'foo'
         )
     file_conf, blocks = split_code_and_text_blocks(filename)
     assert len(blocks) == 2
-    assert blocks[0][0] == "text"
-    assert blocks[1][0] == "code"
+    assert blocks[0].type == "text"
+    assert blocks[1].type == "code"
     assert file_conf == {}
     script_vars = {
         "execute_script": True,
@@ -1043,24 +1090,33 @@ def test_capture_repr(
 
 
 @pytest.mark.parametrize(
-    "caprepr_gallery, caprepr_file, expected_out",
+    "caprepr_gallery, caprepr_file, caprepr_block, expected_out",
     [
-        pytest.param(tuple(), ("__repr__",), "2", id="() --> repr"),
-        pytest.param(("__repr__",), "()", "", id="repr --> ()"),
+        pytest.param(tuple(), "()", '("__repr__",)', "2", id="() --> () --> repr"),
+        pytest.param(tuple(), ("__repr__",), None, "2", id="() --> repr --> None"),
+        pytest.param(("__repr__",), "()", None, "", id="repr --> () --> None"),
+        pytest.param(tuple(), ("__repr__",), "()", "", id="() --> repr --> ()"),
+        pytest.param(("__repr__",), "()", "()", "", id="repr --> () --> ()"),
     ],
 )
-def test_per_file_capture_repr(
+def test_capture_repr_per_file_and_per_block(
     gallery_conf,
     caprepr_gallery,
     caprepr_file,
+    caprepr_block,
     expected_out,
     req_mpl,
     req_pil,
     script_vars,
 ):
-    """Tests that per file capture_repr overrides gallery_conf."""
+    """Tests that per file/block capture_repr overrides gallery_conf."""
     compiler = codeop.Compile()
-    code_block = Block("code", "a=2\n2", 1)
+    caprepr_block = (
+        f"# sphinx_gallery_capture_repr_block={caprepr_block}\n"
+        if caprepr_block
+        else ""
+    )
+    code_block = Block("code", f"{caprepr_block}a=2\n2", 1)
     gallery_conf["capture_repr"] = caprepr_gallery
     file_conf = {"capture_repr": caprepr_file}
     output = sg.execute_code_block(
@@ -1070,7 +1126,7 @@ def test_per_file_capture_repr(
 
 
 def test_ignore_repr_types(gallery_conf, req_mpl, req_pil, script_vars):
-    """Tests output capturing with various capture_repr settings."""
+    """Tests that `ignore_repr_types` correctly ignores the output of a specific type."""
     compiler = codeop.Compile()
     code_block = Block("code", "a=2\na", 1)
     gallery_conf["ignore_repr_types"] = r"int"
@@ -1136,7 +1192,7 @@ def test_reset_module_order_3_param_invalid_when(gallery_conf):
     gallery_conf["reset_modules_order"] = "before"
     with pytest.raises(
         ValueError,
-        match=("3rd parameter in cleanup_3_param " "function signature must be 'when'"),
+        match=("3rd parameter in cleanup_3_param function signature must be 'when'"),
     ):
         _generate_rst(gallery_conf, "plot_test.py", CONTENT)
     assert mock_reset_module.call_count == 0
